@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -15,7 +16,7 @@
 from robot.api import logger
 from robot.utils import (is_dict_like, is_string, is_truthy, plural_or_not,
                          seq2str, seq2str2, type_name, unic, Matcher)
-from robot.utils.asserts import assert_equals
+from robot.utils.asserts import assert_equal
 from robot.version import get_version
 
 
@@ -296,7 +297,7 @@ class _List(object):
         _verify_condition(value in list_, default, msg)
 
     def list_should_not_contain_value(self, list_, value, msg=None):
-        """Fails if the ``value`` is not found from ``list``.
+        """Fails if the ``value`` is found from ``list``.
 
         See `List Should Contain Value` for an explanation of ``msg``.
         """
@@ -384,7 +385,7 @@ class _List(object):
         for index, (item1, item2) in enumerate(zip(list1, list2)):
             name = ' (%s)' % names[index] if index in names else ''
             try:
-                assert_equals(item1, item2, msg='Index %d%s' % (index, name))
+                assert_equal(item1, item2, msg='Index %d%s' % (index, name))
             except AssertionError as err:
                 yield unic(err)
 
@@ -537,8 +538,8 @@ class _Dictionary(object):
     def get_dictionary_keys(self, dictionary):
         """Returns keys of the given ``dictionary``.
 
-        Keys are returned in sorted order. The given ``dictionary`` is never
-        altered by this keyword.
+        If keys are sortable, they are returned in sorted order. The given
+        ``dictionary`` is never altered by this keyword.
 
         Example:
         | ${keys} = | Get Dictionary Keys | ${D3} |
@@ -546,7 +547,11 @@ class _Dictionary(object):
         | ${keys} = ['a', 'b', 'c']
         """
         # TODO: Possibility to disable sorting. Can be handy with OrderedDicts.
-        return sorted(dictionary.keys())
+        keys = dictionary.keys()
+        try:
+            return sorted(keys)
+        except TypeError:
+            return list(keys)
 
     def get_dictionary_values(self, dictionary):
         """Returns values of the given dictionary.
@@ -624,7 +629,7 @@ class _Dictionary(object):
         The given dictionary is never altered by this keyword.
         """
         self.dictionary_should_contain_key(dictionary, key, msg)
-        actual, expected = unicode(dictionary[key]), unicode(value)
+        actual, expected = unic(dictionary[key]), unic(value)
         default = "Value of dictionary key '%s' does not match: %s != %s" % (key, actual, expected)
         _verify_condition(actual == expected, default, msg)
 
@@ -723,7 +728,7 @@ class _Dictionary(object):
     def _yield_dict_diffs(self, keys, dict1, dict2):
         for key in keys:
             try:
-                assert_equals(dict1[key], dict2[key], msg='Key %s' % (key,))
+                assert_equal(dict1[key], dict2[key], msg='Key %s' % (key,))
             except AssertionError as err:
                 yield unic(err)
 
