@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -100,6 +101,9 @@ Examples:
 | ${date} =       | Convert Date | ${date}                 | result_format=%d.%m.%Y |
 | Should Be Equal | ${date}      | 28.05.2014              |
 
+Notice that locale aware directives like ``%b``  do not work correctly with
+Jython on non-English locales: http://bugs.jython.org/issue2285
+
 == Python datetime ==
 
 Python's standard
@@ -306,7 +310,7 @@ import re
 
 from robot.version import get_version
 from robot.utils import (elapsed_time_to_string, is_falsy, is_number,
-                         is_string, secs_to_timestr, timestr_to_secs,
+                         is_string, roundup, secs_to_timestr, timestr_to_secs,
                          type_name, IRONPYTHON)
 
 __version__ = get_version()
@@ -535,7 +539,7 @@ class Date(object):
         # https://github.com/IronLanguages/main/issues/1170
         # Also Jython had similar problems, but they seem to be fixed in 2.7.
         dt = datetime.fromtimestamp(secs)
-        return dt.replace(microsecond=int(round(secs % 1 * 1e6)))
+        return dt.replace(microsecond=roundup(secs % 1 * 1e6))
 
     def _string_to_datetime(self, ts, input_format):
         if not input_format:
@@ -599,7 +603,7 @@ class Date(object):
     def _convert_to_timestamp(self, dt, millis=True):
         if not millis:
             return dt.strftime('%Y-%m-%d %H:%M:%S')
-        ms = int(round(dt.microsecond / 1000.0))
+        ms = roundup(dt.microsecond / 1000.0)
         if ms == 1000:
             dt += timedelta(seconds=1)
             ms = 0
@@ -625,7 +629,7 @@ class Date(object):
 class Time(object):
 
     def __init__(self, time):
-        self.seconds = self._convert_time_to_seconds(time)
+        self.seconds = float(self._convert_time_to_seconds(time))
 
     def _convert_time_to_seconds(self, time):
         if isinstance(time, timedelta):
@@ -644,7 +648,7 @@ class Time(object):
             result_converter = getattr(self, '_convert_to_%s' % format.lower())
         except AttributeError:
             raise ValueError("Unknown format '%s'." % format)
-        seconds = self.seconds if millis else round(self.seconds)
+        seconds = self.seconds if millis else float(roundup(self.seconds))
         return result_converter(seconds, millis)
 
     def _convert_to_number(self, seconds, millis=True):
