@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,23 +13,23 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
-import codecs
 import fnmatch
 import glob
+import io
 import os
 import shutil
-import subprocess
 import sys
 import tempfile
 import time
 
 from robot.version import get_version
 from robot.api import logger
-from robot.utils import (abspath, ConnectionCache, decode_output, del_env_var,
+from robot.utils import (abspath, ConnectionCache, console_decode, del_env_var,
                          get_env_var, get_env_vars, get_time, is_truthy,
                          is_unicode, normpath, parse_time, plural_or_not,
                          secs_to_timestamp, secs_to_timestr, seq2str,
-                         set_env_var, timestr_to_secs, unic)
+                         set_env_var, timestr_to_secs, unic, CONSOLE_ENCODING,
+                         IRONPYTHON, JYTHON, PY2, SYSTEM_ENCODING, WINDOWS)
 
 __version__ = get_version()
 PROCESSES = ConnectionCache('No active processes.')
@@ -51,7 +52,6 @@ class OperatingSystem(object):
     - `Path separators`
     - `Pattern matching`
     - `Tilde expansion`
-    - `Process library`
     - `Boolean arguments`
     - `Example`
     - `Shortcuts`
@@ -100,19 +100,6 @@ class OperatingSystem(object):
     Tilde expansion is a new feature in Robot Framework 2.8. The ``~username``
     form does not work on Jython
 
-    = Process library =
-
-    [http://robotframework.org/robotframework/latest/libraries/Process.html|
-    Process library] that was added in Robot Framework 2.8 provides much more
-    flexible keywords than this library for running processes in general and
-    for starting processes on background in particular. Keywords provided
-    by the Process library are thus recommended instead of `Run`, `Start
-    Process`, and other related keywords in this library.
-
-    `Start Process`, `Read Process Output`, `Switch Process`, `Stop Process`,
-    and `Stop All Processes` were officially deprecated in Robot Framework 2.9.
-    They will be removed altogether in the future.
-
     = Boolean arguments =
 
     Some keywords accept arguments that are handled as Boolean values true or
@@ -186,9 +173,10 @@ class OperatingSystem(object):
         | Should Be Equal    | ${stdout} | TEST PASSED |
         | File Should Be Empty | /tmp/stderr.txt |
 
-        *TIP:* `Run Process` keyword provided by the `Process library` supports
-        better process configuration and is generally recommended as a
-        replacement for this keyword.
+        *TIP:* `Run Process` keyword provided by the
+        [http://robotframework.org/robotframework/latest/libraries/Process.html|
+        Process library] supports better process configuration and is generally
+        recommended as a replacement for this keyword.
         """
         return self._run(command)[1]
 
@@ -212,9 +200,10 @@ class OperatingSystem(object):
         See `Run` and `Run And Return RC And Output` if you need to get the
         output of the executed command.
 
-        *TIP:* `Run Process` keyword provided by the `Process library` supports
-        better process configuration and is generally recommended as a
-        replacement for this keyword.
+        *TIP:* `Run Process` keyword provided by the
+        [http://robotframework.org/robotframework/latest/libraries/Process.html|
+        Process library] supports better process configuration and is generally
+        recommended as a replacement for this keyword.
         """
         return self._run(command)[0]
 
@@ -233,9 +222,10 @@ class OperatingSystem(object):
         | Should Be Equal      | ${stdout}       | TEST PASSED |
         | File Should Be Empty | /tmp/stderr.txt |
 
-        *TIP:* `Run Process` keyword provided by the `Process library` supports
-        better process configuration and is generally recommended as a
-        replacement for this keyword.
+        *TIP:* `Run Process` keyword provided by the
+        [http://robotframework.org/robotframework/latest/libraries/Process.html|
+        Process library] supports better process configuration and is generally
+        recommended as a replacement for this keyword.
         """
         return self._run(command)
 
@@ -246,117 +236,6 @@ class OperatingSystem(object):
         rc = process.close()
         return rc, stdout
 
-    def start_process(self, command, stdin=None, alias=None):
-        """*DEPRECATED.* Use keywords in the `Process library` instead.
-
-        Starts the given command as a background process.
-
-        Starts the process in background and sets it as the active process.
-        `Read Process Output` or `Stop Process` keywords affect this process
-        unless `Switch Process` is used in between.
-
-        If the command needs input through the standard input stream,
-        it can be defined with the ``stdin`` argument.  It is not
-        possible to give input to the command later. Possible command
-        line arguments must be given as part of the command like
-        ``/tmp/script.sh arg1 arg2``.
-
-        Returns the index of this process. Indexing starts from 1, and indices
-        can be used to switch between processes using `Switch Process` keyword.
-        `Stop All Processes` can be used to reset indexing.
-
-        The optional ``alias`` is a name for this process that may be used with
-        `Switch Process` instead of the returned index.
-
-        The standard error stream is redirected to the standard input
-        stream automatically. This is done for the same reasons as with `Run`
-        keyword, but redirecting is done when the process is started and not
-        by adding ``2>&1`` to the command.
-
-        Example:
-        | Start Process  | /path/longlasting.sh |
-        | Do Something   |                      |
-        | ${output} =    | Read Process Output  |
-        | Should Contain | ${output}            | Expected text |
-        | [Teardown]     | Stop All Processes   |
-        """
-        process = _Process2(command, stdin)
-        self._info("Running command '%s'." % process)
-        return PROCESSES.register(process, alias)
-
-    def switch_process(self, index_or_alias):
-        """*DEPRECATED.* Use keywords in the `Process library` instead.
-
-        Switches the active process to the specified process.
-
-        New active process can be specified either using an index or an alias.
-        Indices are return values from `Start Process` and aliases can be
-        given to that keyword.
-
-        Example:
-        | Start Process  | /path/script.sh arg  | alias=1st process |
-        | ${2nd} =       | Start Process        | /path/script2.sh |
-        | Switch Process | 1st process          |
-        | ${out1} =      | Read Process Output  |
-        | Switch Process | ${2nd}               |
-        | ${out2} =      | Read Process Output  |
-        | Log Many       | 1st process: ${out1} | 2nd process: ${out1} |
-        | [Teardown]     | Stop All Processes   |
-        """
-        PROCESSES.switch(index_or_alias)
-
-    def read_process_output(self):
-        """*DEPRECATED.* Use keywords in the `Process library` instead.
-
-        Waits for a process to finish and returns its output.
-
-        This keyword waits for a process started with `Start Process` to end
-        and then returns all output it has produced. The returned output
-        contains everything the process has written into the standard output
-        and error streams.
-
-        There is no need to use `Stop Process` after using this keyword.
-        Trying to read from an already stopped process fails.
-
-        Note that although the process is finished, it still stays as the
-        active process. Use `Switch Process` to switch the active process or
-        `Stop All Processes` to reset the list of started processes.
-        """
-        output = PROCESSES.current.read()
-        PROCESSES.current.close()
-        return output
-
-    def stop_process(self):
-        """*DEPRECATED.* Use keywords in the `Process library` instead.
-
-        Closes the standard output stream of the process.
-
-        This keyword does not actually stop the process nor even wait for it
-        to terminate. Only thing it does is closing the standard output stream
-        of the process. Depending on the process that may terminate it but
-        that is not guaranteed. Use `Read Process Output` instead if you need
-        to wait for the process to complete.
-
-        This keyword operates the active process similarly as `Read Process
-        Output`. Stopping an already stopped process is not an error.
-        """
-        PROCESSES.current.close()
-
-    def stop_all_processes(self):
-        """*DEPRECATED.* Use keywords in the `Process library` instead.
-
-        Closes the standard output of all the processes and resets the process
-        list.
-
-        Exactly like `Stop Process`, this keyword does not actually stop
-        processes nor even wait for them to terminate.
-
-        This keyword resets the indexing that `Start Process` uses. All aliases
-        are also deleted. It does not matter have some of the processes
-        already been closed or not.
-        """
-        PROCESSES.close_all()
-
     def get_file(self, path, encoding='UTF-8', encoding_errors='strict'):
         """Returns the contents of a specified file.
 
@@ -365,22 +244,43 @@ class OperatingSystem(object):
         See also `Get Binary File`.
 
         ``encoding`` defines the encoding of the file. The default value is
-        UTF-8, which means that UTF-8 and ASCII-encoded files are read
-        correctly.
+        ``UTF-8``, which means that UTF-8 and ASCII encoded files are read
+        correctly. In addition to the encodings supported by the underlying
+        Python implementation, the following special encoding values can be
+        used:
+
+        - ``SYSTEM``: Use the default system encoding.
+        - ``CONSOLE``: Use the console encoding. Outside Windows this is same
+          as the system encoding.
 
         ``encoding_errors`` argument controls what to do if decoding some bytes
         fails. All values accepted by ``decode`` method in Python are valid, but
         in practice the following values are most useful:
 
-        - ``strict``: fail if characters cannot be decoded (default)
-        - ``ignore``: ignore characters that cannot be decoded
-        - ``replace``: replace characters that cannot be decoded with
-          a replacement character
+        - ``strict``: Fail if characters cannot be decoded (default).
+        - ``ignore``: Ignore characters that cannot be decoded.
+        - ``replace``: Replace characters that cannot be decoded with
+          a replacement character.
 
-        ``encoding_errors`` argument is new in Robot Framework 2.8.5.
+        ``encoding_errors`` argument was added in Robot Framework 2.8.5 and the
+        support for ``SYSTEM`` and ``CONSOLE`` encodings in Robot Framework 3.0.
         """
-        content = self.get_binary_file(path)
-        return unicode(content, encoding, encoding_errors).replace('\r\n', '\n')
+        path = self._absnorm(path)
+        self._link("Getting file '%s'.", path)
+        encoding = self._map_encoding(encoding)
+        if IRONPYTHON:
+            # https://github.com/IronLanguages/main/issues/1233
+            with open(path) as f:
+                content = f.read().decode(encoding, encoding_errors)
+        else:
+            with io.open(path, encoding=encoding, errors=encoding_errors,
+                         newline='') as f:
+                content = f.read()
+        return content.replace('\r\n', '\n')
+
+    def _map_encoding(self, encoding):
+        return {'SYSTEM': SYSTEM_ENCODING,
+                'CONSOLE': CONSOLE_ENCODING}.get(encoding.upper(), encoding)
 
     def get_binary_file(self, path):
         """Returns the contents of a specified file.
@@ -391,7 +291,7 @@ class OperatingSystem(object):
         path = self._absnorm(path)
         self._link("Getting file '%s'.", path)
         with open(path, 'rb') as f:
-            return f.read()
+            return bytes(f.read())
 
     def grep_file(self, path, pattern, encoding='UTF-8', encoding_errors='strict'):
         """Returns the lines of the specified file that match the ``pattern``.
@@ -423,7 +323,7 @@ class OperatingSystem(object):
         lines = []
         total_lines = 0
         self._link("Reading file '%s'.", path)
-        with codecs.open(path, encoding=encoding, errors=encoding_errors) as f:
+        with io.open(path, encoding=encoding, errors=encoding_errors) as f:
             for line in f.readlines():
                 total_lines += 1
                 line = line.rstrip('\r\n')
@@ -439,6 +339,9 @@ class OperatingSystem(object):
         just use `Get File` and the built-in keyword `Log` with the desired
         level.
 
+        See `Get File` for more information about ``encoding`` and
+        ``encoding_errors`` arguments.
+
         ``encoding_errors`` argument is new in Robot Framework 2.8.5.
         """
         content = self.get_file(path, encoding, encoding_errors)
@@ -647,19 +550,26 @@ class OperatingSystem(object):
     def create_file(self, path, content='', encoding='UTF-8'):
         """Creates a file with the given content and encoding.
 
-        If the directory where to create file does not exist it, and possible
-        intermediate missing directories, are created.
+        If the directory for the file does not exist, it is created, along
+        with missing intermediate directories.
+
+        See `Get File` for more information about possible ``encoding`` values,
+        including special values ``SYSTEM`` and ``CONSOLE``.
 
         Examples:
         | Create File | ${dir}/example.txt | Hello, world!      |         |
-        | Create File | ${path}            | Hyv\\xe4 esimerkki | latin-1 |
+        | Create File | ${path}            | Hyv\\xe4 esimerkki | Latin-1 |
+        | Create File | /tmp/foo.txt       | ${content}         | SYSTEM  |
 
         Use `Append To File` if you want to append to an existing file
         and `Create Binary File` if you need to write bytes without encoding.
         `File Should Not Exist` can be used to avoid overwriting existing
         files.
+
+        The support for ``SYSTEM`` and ``CONSOLE`` encodings is new in Robot
+        Framework 3.0.
         """
-        path = self._write_to_file(path, content, encoding)
+        path = self._write_to_file(path, content, self._map_encoding(encoding))
         self._link("Created file '%s'.", path)
 
     def _write_to_file(self, path, content, encoding=None, mode='w'):
@@ -683,8 +593,8 @@ class OperatingSystem(object):
 
         Byte strings, and possible other types, are written to the file as is.
 
-        If the directory where to create file does not exist it, and possible
-        intermediate missing directories, are created.
+        If the directory for the file does not exist, it is created, along
+        with missing intermediate directories.
 
         Examples:
         | Create Binary File | ${dir}/example.png | ${image content}     |
@@ -697,12 +607,12 @@ class OperatingSystem(object):
         New in Robot Framework 2.8.5.
         """
         if is_unicode(content):
-            content = ''.join(chr(ord(c)) for c in content)
+            content = bytes(bytearray(ord(c) for c in content))
         path = self._write_to_file(path, content)
         self._link("Created binary file '%s'.", path)
 
     def append_to_file(self, path, content, encoding='UTF-8'):
-        """Appends the given contend to the specified file.
+        """Appends the given content to the specified file.
 
         If the file does not exists, this keyword works exactly the same
         way as `Create File`.
@@ -980,7 +890,12 @@ class OperatingSystem(object):
         """
         source, destination \
             = self._prepare_copy_and_move_directory(source, destination)
-        shutil.copytree(source, destination)
+        try:
+            shutil.copytree(source, destination)
+        except shutil.Error:
+            # https://github.com/robotframework/robotframework/issues/2321
+            if not (WINDOWS and JYTHON):
+                raise
         self._link("Copied directory from '%s' to '%s'.", source, destination)
 
     def _prepare_copy_and_move_directory(self, source, destination):
@@ -1501,10 +1416,7 @@ class OperatingSystem(object):
         self._log(msg, 'WARN')
 
     def _log(self, msg, level):
-        if logger:
-            logger.write(msg, level)
-        else:
-            print '*%s* %s' % (level, msg)
+        logger.write(msg, level)
 
 
 class _Process:
@@ -1545,7 +1457,7 @@ class _Process:
         return self._encode_to_file_system(command)
 
     def _encode_to_file_system(self, string):
-        enc = sys.getfilesystemencoding()
+        enc = sys.getfilesystemencoding() if PY2 else None
         return string.encode(enc) if enc else string
 
     def _process_output(self, output):
@@ -1553,28 +1465,4 @@ class _Process:
             output = output.replace('\r\n', '\n')
         if output.endswith('\n'):
             output = output[:-1]
-        return decode_output(output, force=True)
-
-
-class _Process2(_Process):
-
-    def __init__(self, command, input_):
-        self._command = self._process_command(command)
-        p = subprocess.Popen(self._command, shell=True, stdin=subprocess.PIPE,
-                             stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
-                             close_fds=os.sep=='/')
-        stdin, self.stdout = p.stdin, p.stdout
-        if input_:
-            stdin.write(input_)
-        stdin.close()
-        self.closed = False
-
-    def read(self):
-        if self.closed:
-            raise RuntimeError('Cannot read from a closed process.')
-        return self._process_output(self.stdout.read())
-
-    def close(self):
-        if not self.closed:
-            self.stdout.close()
-            self.closed = True
+        return console_decode(output, force=True)
