@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -18,10 +19,10 @@ import subprocess
 import time
 import signal as signal_module
 
-from robot.utils import (ConnectionCache, abspath, cmdline2list,
-                         encode_to_system, decode_output, is_list_like,
-                         is_truthy, secs_to_timestr, timestr_to_secs,
-                         IRONPYTHON, JYTHON)
+from robot.utils import (ConnectionCache, abspath, cmdline2list, console_decode,
+                         is_list_like, is_truthy, NormalizedDict, py2to3,
+                         secs_to_timestr, system_decode, system_encode,
+                         timestr_to_secs, IRONPYTHON, JYTHON, WINDOWS)
 from robot.version import get_version
 from robot.api import logger
 
@@ -52,7 +53,6 @@ class Process(object):
     - `Active process`
     - `Result object`
     - `Boolean arguments`
-    - `Using with OperatingSystem library`
     - `Example`
     - `Shortcuts`
     - `Keywords`
@@ -94,6 +94,7 @@ class Process(object):
     | env:<name> | Overrides the named environment variable(s) only.     |
     | stdout     | Path of a file where to write standard output.        |
     | stderr     | Path of a file where to write standard error.         |
+    | output_encoding | Encoding to use when reading command outputs.    |
     | alias      | Alias given to the process.                           |
 
     Note that because ``**configuration`` is passed using ``name=value`` syntax,
@@ -170,7 +171,9 @@ class Process(object):
     the standard output by using ``stderr=STDOUT``.
 
     Regardless are outputs redirected to files or not, they are accessible
-    through the `result object` returned when the process ends.
+    through the `result object` returned when the process ends. Commands are
+    expected to write outputs using the console encoding, but `output encoding`
+    can be configured using the ``output_encoding`` argument if needed.
 
     Examples:
     | ${result} = | `Run Process` | program | stdout=${TEMPDIR}/stdout.txt | stderr=${TEMPDIR}/stderr.txt |
@@ -181,6 +184,29 @@ class Process(object):
     Note that the created output files are not automatically removed after
     the test run. The user is responsible to remove them if needed.
 
+    == Output encoding ==
+
+    Executed commands are, by default, expected to write outputs to the
+    `standard output and error streams` using the encoding used by the
+    system console. If the command uses some other encoding, that can be
+    configured using the ``output_encoding`` argument. This is especially
+    useful on Windows where the console uses a different encoding than rest
+    of the system, and many commands use the general system encoding instead
+    of the console encoding.
+
+    The value used with the ``output_encoding`` argument must be a valid
+    encoding and must match the encoding actually used by the command. As a
+    convenience, it is possible to use strings ``CONSOLE`` and ``SYSTEM``
+    to specify that the console or system encoding is used, respectively.
+    If produced outputs use different encoding then configured, values got
+    through the `result object` will be invalid.
+
+    Examples:
+    | `Start Process` | program | output_encoding=UTF-8 |
+    | `Run Process`   | program | stdout=${path} | output_encoding=SYSTEM |
+
+    The support to set output encoding is new in Robot Framework 3.0.
+
     == Alias ==
 
     A custom name given to the process that can be used when selecting the
@@ -253,23 +279,6 @@ class Process(object):
     ``false``, were considered true. Additionally, ``no`` is considered false
     only in Robot Framework 2.9 and newer.
 
-    = Using with OperatingSystem library =
-
-    The OperatingSystem library also contains keywords for running processes.
-    They are not as flexible as the keywords provided by this library, and
-    thus not recommended to be used anymore. They may eventually even be
-    deprecated.
-
-    There is a name collision because both of these libraries have
-    `Start Process` and `Switch Process` keywords. This is handled so that
-    if both libraries are imported, the keywords in the Process library are
-    used by default. If there is a need to use the OperatingSystem variants,
-    it is possible to use `OperatingSystem.Start Process` syntax or use
-    the BuiltIn keyword `Set Library Search Order` to change the priority.
-
-    Other keywords in the OperatingSystem library can be used freely with
-    keywords in the Process library.
-
     = Example =
 
     | ***** Settings *****
@@ -345,35 +354,25 @@ class Process(object):
         for related examples.
 
         Makes the started process new `active process`. Returns an identifier
-        that can be used as a handle to active the started process if needed.
+        that can be used as a handle to activate the started process if needed.
 
         Starting from Robot Framework 2.8.5, processes are started so that
         they create a new process group. This allows sending signals to and
         terminating also possible child processes. This is not supported by
         Jython in general nor by Python versions prior to 2.7 on Windows.
         """
-        config = ProcessConfig(**configuration)
-        command = self._get_command(command, arguments, config.shell)
-        self._log_start(command, config)
-        process = subprocess.Popen(command, **config.full_config)
-        self._results[process] = ExecutionResult(process,
-                                                 config.stdout_stream,
-                                                 config.stderr_stream)
-        return self._processes.register(process, alias=config.alias)
-
-    def _get_command(self, command, args, use_shell):
-        command = [encode_to_system(item) for item in [command] + list(args)]
-        if not use_shell:
-            return command
-        if args:
-            return subprocess.list2cmdline(command)
-        return command[0]
+        conf = ProcessConfiguration(**configuration)
+        command = conf.get_command(command, list(arguments))
+        self._log_start(command, conf)
+        process = subprocess.Popen(command, **conf.popen_config)
+        self._results[process] = ExecutionResult(process, **conf.result_config)
+        return self._processes.register(process, alias=conf.alias)
 
     def _log_start(self, command, config):
         if is_list_like(command):
             command = self.join_command_line(command)
-        logger.info('Starting process:\n%s' % command)
-        logger.debug('Process configuration:\n%s' % config)
+        logger.info(u'Starting process:\n%s' % system_decode(command))
+        logger.debug(u'Process configuration:\n%s' % config)
 
     def is_process_running(self, handle=None):
         """Checks is the process running or not.
@@ -771,11 +770,12 @@ class Process(object):
 
 class ExecutionResult(object):
 
-    def __init__(self, process, stdout, stderr, rc=None):
+    def __init__(self, process, stdout, stderr, rc=None, output_encoding=None):
         self._process = process
         self.stdout_path = self._get_path(stdout)
         self.stderr_path = self._get_path(stderr)
         self.rc = rc
+        self._output_encoding = output_encoding
         self._stdout = None
         self._stderr = None
         self._custom_streams = [stream for stream in (stdout, stderr)
@@ -807,24 +807,27 @@ class ExecutionResult(object):
 
     def _read_stream(self, stream_path, stream):
         if stream_path:
-            stream = open(stream_path, 'r')
+            stream = open(stream_path, 'rb')
         elif not self._is_open(stream):
             return ''
         try:
-            return self._format_output(stream.read())
+            content = stream.read()
         except IOError:  # http://bugs.jython.org/issue2218
             return ''
         finally:
             if stream_path:
                 stream.close()
+        return self._format_output(content)
 
     def _is_open(self, stream):
         return stream and not stream.closed
 
     def _format_output(self, output):
+        output = console_decode(output, self._output_encoding, force=True)
+        output = output.replace('\r\n', '\n')
         if output.endswith('\n'):
             output = output[:-1]
-        return decode_output(output, force=True)
+        return output
 
     def close_streams(self):
         standard_streams = self._get_and_read_standard_streams(self._process)
@@ -844,15 +847,17 @@ class ExecutionResult(object):
         return '<result object with rc %d>' % self.rc
 
 
-class ProcessConfig(object):
+@py2to3
+class ProcessConfiguration(object):
 
     def __init__(self, cwd=None, shell=False, stdout=None, stderr=None,
-                 alias=None, env=None, **rest):
+                 output_encoding='CONSOLE', alias=None, env=None, **rest):
         self.cwd = self._get_cwd(cwd)
         self.stdout_stream = self._new_stream(stdout)
         self.stderr_stream = self._get_stderr(stderr, stdout, self.stdout_stream)
         self.shell = is_truthy(shell)
         self.alias = alias
+        self.output_encoding = output_encoding
         self.env = self._construct_env(env, rest)
 
     def _get_cwd(self, cwd):
@@ -874,27 +879,46 @@ class ProcessConfig(object):
         return self._new_stream(stderr)
 
     def _construct_env(self, env, extra):
+        env = self._get_initial_env(env, extra)
+        if env is None:
+            return None
+        if WINDOWS:
+            env = NormalizedDict(env, spaceless=False)
+        self._add_to_env(env, extra)
+        if WINDOWS:
+            env = dict((key.upper(), env[key]) for key in env)
+        return env
+
+    def _get_initial_env(self, env, extra):
         if env:
-            env = dict((encode_to_system(k), encode_to_system(v))
-                       for k, v in env.items())
+            return dict((system_encode(k), system_encode(env[k])) for k in env)
+        if extra:
+            return os.environ.copy()
+        return None
+
+    def _add_to_env(self, env, extra):
         for key in extra:
             if not key.startswith('env:'):
                 raise RuntimeError("Keyword argument '%s' is not supported by "
                                    "this keyword." % key)
-            if env is None:
-                env = os.environ.copy()
-            env[encode_to_system(key[4:])] = encode_to_system(extra[key])
-        return env
+            env[system_encode(key[4:])] = system_encode(extra[key])
+
+    def get_command(self, command, arguments):
+        command = [system_encode(item) for item in [command] + arguments]
+        if not self.shell:
+            return command
+        if arguments:
+            return subprocess.list2cmdline(command)
+        return command[0]
 
     @property
-    def full_config(self):
+    def popen_config(self):
         config = {'stdout': self.stdout_stream,
                   'stderr': self.stderr_stream,
                   'stdin': subprocess.PIPE,
                   'shell': self.shell,
                   'cwd': self.cwd,
-                  'env': self.env,
-                  'universal_newlines': True}
+                  'env': self.env}
         if not JYTHON:
             self._add_process_group_config(config)
         return config
@@ -905,12 +929,24 @@ class ProcessConfig(object):
         if hasattr(subprocess, 'CREATE_NEW_PROCESS_GROUP'):
             config['creationflags'] = subprocess.CREATE_NEW_PROCESS_GROUP
 
-    def __str__(self):
-        return encode_to_system("""\
-cwd = %s
-stdout_stream = %s
-stderr_stream = %s
-shell = %r
-alias = %s
-env = %r""" % (self.cwd, self.stdout_stream, self.stderr_stream,
-               self.shell, self.alias, self.env))
+    @property
+    def result_config(self):
+        return {'stdout': self.stdout_stream,
+                'stderr': self.stderr_stream,
+                'output_encoding': self.output_encoding}
+
+    def __unicode__(self):
+        return """\
+cwd:     %s
+shell:   %s
+stdout:  %s
+stderr:  %s
+alias:   %s
+env:     %s""" % (self.cwd, self.shell, self._stream_name(self.stdout_stream),
+                  self._stream_name(self.stderr_stream), self.alias, self.env)
+
+    def _stream_name(self, stream):
+        if hasattr(stream, 'name'):
+            return stream.name
+        return {subprocess.PIPE: 'PIPE',
+                subprocess.STDOUT: 'STDOUT'}.get(stream, stream)
