@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,12 +13,17 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
-import httplib
+try:
+    import httplib
+    import xmlrpclib
+except ImportError:  # Py3
+    import http.client as httplib
+    import xmlrpc.client as xmlrpclib
 import re
 import socket
 import sys
 import time
-import xmlrpclib
+
 try:
     from xml.parsers.expat import ExpatError
 except ImportError:   # No expat in IronPython 2.7
@@ -26,7 +32,8 @@ except ImportError:   # No expat in IronPython 2.7
 
 from robot.errors import RemoteError
 from robot.utils import (is_bytes, is_dict_like, is_list_like, is_number,
-                         is_string, timestr_to_secs, unic, DotDict, IRONPYTHON)
+                         is_string, timestr_to_secs, unic, DotDict, IRONPYTHON,
+                         JYTHON)
 
 
 class Remote(object):
@@ -71,6 +78,12 @@ class Remote(object):
         except TypeError:
             return ['*args']
 
+    def get_keyword_tags(self, name):
+        try:
+            return self._client.get_keyword_tags(name)
+        except TypeError:
+            return None
+
     def get_keyword_documentation(self, name):
         try:
             return self._client.get_keyword_documentation(name)
@@ -94,37 +107,38 @@ class ArgumentCoercer(object):
     non_ascii = re.compile('[\x80-\xff]')
 
     def coerce(self, argument):
-        for handles, handle in [(self._is_string, self._handle_string),
-                                (self._is_number, self._pass_through),
-                                (is_dict_like, self._coerce_dict),
-                                (is_list_like, self._coerce_list),
-                                (lambda arg: True, self._to_string)]:
+        for handles, handler in [(is_string, self._handle_string),
+                                 (is_bytes, self._handle_bytes),
+                                 (is_number, self._pass_through),
+                                 (is_dict_like, self._coerce_dict),
+                                 (is_list_like, self._coerce_list),
+                                 (lambda arg: True, self._to_string)]:
             if handles(argument):
-                return handle(argument)
-
-    def _is_string(self, arg):
-        return is_string(arg)
-
-    def _is_number(self, arg):
-        return is_number(arg)
+                return handler(argument)
 
     def _handle_string(self, arg):
-        if self._contains_binary(arg):
-            return self._handle_binary(arg)
+        if self._string_contains_binary(arg):
+            return self._handle_binary_in_string(arg)
         return arg
 
-    def _contains_binary(self, arg):
+    def _string_contains_binary(self, arg):
         return (self.binary.search(arg) or
-                is_bytes(arg) and not IRONPYTHON and
-                self.non_ascii.search(arg))
+                is_bytes(arg) and self.non_ascii.search(arg))
 
-    def _handle_binary(self, arg):
+    def _handle_binary_in_string(self, arg):
         try:
-            arg = str(arg)
+            if not is_bytes(arg):
+                arg = arg.encode('ASCII')
         except UnicodeError:
             raise ValueError('Cannot represent %r as binary.' % arg)
         return xmlrpclib.Binary(arg)
 
+    def _handle_bytes(self, arg):
+        # http://bugs.jython.org/issue2429
+        if IRONPYTHON or JYTHON:
+            arg = str(arg)
+        return xmlrpclib.Binary(arg)
+
     def _pass_through(self, arg):
         return arg
 
@@ -158,10 +172,10 @@ class RemoteResult(object):
         if not (is_dict_like(result) and 'status' in result):
             raise RuntimeError('Invalid remote result dictionary: %s' % result)
         self.status = result['status']
-        self.output = self._get(result, 'output')
+        self.output = unic(self._get(result, 'output'))
         self.return_ = self._get(result, 'return')
-        self.error = self._get(result, 'error')
-        self.traceback = self._get(result, 'traceback')
+        self.error = unic(self._get(result, 'error'))
+        self.traceback = unic(self._get(result, 'traceback'))
         self.fatal = bool(self._get(result, 'fatal', False))
         self.continuable = bool(self._get(result, 'continuable', False))
 
@@ -171,7 +185,7 @@ class RemoteResult(object):
 
     def _convert(self, value):
         if isinstance(value, xmlrpclib.Binary):
-            return str(value)
+            return bytes(value.data)
         if is_dict_like(value):
             return DotDict((k, self._convert(v)) for k, v in value.items())
         if is_list_like(value):
@@ -198,6 +212,12 @@ class XmlRpcRemoteClient(object):
         except xmlrpclib.Error:
             raise TypeError
 
+    def get_keyword_tags(self, name):
+        try:
+            return self._server.get_keyword_tags(name)
+        except xmlrpclib.Error:
+            raise TypeError
+
     def get_keyword_documentation(self, name):
         try:
             return self._server.get_keyword_documentation(name)
