@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -18,7 +19,8 @@ from random import randint
 from string import ascii_lowercase, ascii_uppercase, digits
 
 from robot.api import logger
-from robot.utils import is_bytes, is_string, is_truthy, is_unicode, lower, unic
+from robot.utils import (is_bytes, is_string, is_truthy, is_unicode, lower,
+                         unic, PY3)
 from robot.version import get_version
 
 
@@ -94,10 +96,8 @@ class String(object):
         on character or integer sequences. Use `Decode Bytes To String` if you
         need to convert byte strings to Unicode strings and `Convert To String`
         in ``BuiltIn`` if you need to convert arbitrary objects to Unicode.
-
-        New in Robot Framework 2.7.7.
         """
-        return string.encode(encoding, errors)
+        return bytes(string.encode(encoding, errors))
 
     def decode_bytes_to_string(self, bytes, encoding, errors='strict'):
         """Decodes the given ``bytes`` to a Unicode string using the given ``encoding``.
@@ -118,9 +118,9 @@ class String(object):
         Use `Encode String To Bytes` if you need to convert Unicode strings to
         byte strings, and `Convert To String` in ``BuiltIn`` if you need to
         convert arbitrary objects to Unicode strings.
-
-        New in Robot Framework 2.7.7.
         """
+        if PY3 and is_unicode(bytes):
+            raise TypeError('Can not decode strings on Python 3.')
         return bytes.decode(encoding, errors)
 
     def get_line_count(self, string):
@@ -453,8 +453,6 @@ class String(object):
 
         Example:
         | @{characters} = | Split String To Characters | ${string} |
-
-        New in Robot Framework 2.7.
         """
         return list(string)
 
@@ -507,7 +505,7 @@ class String(object):
                             ('[NUMBERS]', digits)]:
             chars = chars.replace(name, value)
         maxi = len(chars) - 1
-        return ''.join(chars[randint(0, maxi)] for _ in xrange(length))
+        return ''.join(chars[randint(0, maxi)] for _ in range(length))
 
     def get_substring(self, string, start, end=None):
         """Returns a substring from ``start`` index to ``end`` index.
@@ -527,12 +525,52 @@ class String(object):
         end = self._convert_to_index(end, 'end')
         return string[start:end]
 
+    def strip_string(self, string, mode='both', characters=None):
+        """Remove leading and/or trailing whitespaces from the given string.
+
+        ``mode`` is either ``left`` to remove leading characters, ``right`` to
+        remove trailing characters, ``both`` (default) to remove the
+        characters from both sides of the string or ``none`` to return the
+        unmodified string.
+
+        If the optional ``characters`` is given, it must be a string and the
+        characters in the string will be stripped in the string. Please note,
+        that this is not a substring to be removed but a list of characters,
+        see the example below.
+
+        Examples:
+        | ${stripped}=  | Strip String | ${SPACE}Hello${SPACE} | |
+        | Should Be Equal | ${stripped} | Hello | |
+        | ${stripped}=  | Strip String | ${SPACE}Hello${SPACE} | mode=left |
+        | Should Be Equal | ${stripped} | Hello${SPACE} | |
+        | ${stripped}=  | Strip String | aabaHelloeee | characters=abe |
+        | Should Be Equal | ${stripped} | Hello | |
+
+        New in Robot Framework 3.0.
+        """
+        try:
+            method = {'BOTH': string.strip,
+                      'LEFT': string.lstrip,
+                      'RIGHT': string.rstrip,
+                      'NONE': lambda characters: string}[mode.upper()]
+        except KeyError:
+            raise ValueError("Invalid mode '%s'." % mode)
+        return method(characters)
+
     def should_be_string(self, item, msg=None):
         """Fails if the given ``item`` is not a string.
 
-        This keyword passes regardless is the ``item`` is a Unicode string or
-        a byte string. Use `Should Be Unicode String` or `Should Be Byte
-        String` if you want to restrict the string type.
+        With Python 2, except with IronPython, this keyword passes regardless
+        is the ``item`` a Unicode string or a byte string. Use `Should Be
+        Unicode String` or `Should Be Byte String` if you want to restrict
+        the string type. Notice that with Python 2, except with IronPython,
+        ``'string'`` creates a byte string and ``u'unicode'`` must be used to
+        create a Unicode string.
+
+        With Python 3 and IronPython, this keyword passes if the string is
+        a Unicode string but fails if it is bytes. Notice that with both
+        Python 3 and IronPython, ``'string'`` creates a Unicode string, and
+        ``b'bytes'`` must be used to create a byte string.
 
         The default error message can be overridden with the optional
         ``msg`` argument.
@@ -543,6 +581,9 @@ class String(object):
     def should_not_be_string(self, item, msg=None):
         """Fails if the given ``item`` is a string.
 
+        See `Should Be String` for more details about Unicode strings and byte
+        strings.
+
         The default error message can be overridden with the optional
         ``msg`` argument.
         """
@@ -554,12 +595,11 @@ class String(object):
 
         Use `Should Be Byte String` if you want to verify the ``item`` is a
         byte string, or `Should Be String` if both Unicode and byte strings
-        are fine.
+        are fine. See `Should Be String` for more details about Unicode
+        strings and byte strings.
 
         The default error message can be overridden with the optional
         ``msg`` argument.
-
-        New in Robot Framework 2.7.7.
         """
         if not is_unicode(item):
             self._fail(msg, "'%s' is not a Unicode string.", item)
@@ -569,12 +609,11 @@ class String(object):
 
         Use `Should Be Unicode String` if you want to verify the ``item`` is a
         Unicode string, or `Should Be String` if both Unicode and byte strings
-        are fine.
+        are fine. See `Should Be String` for more details about Unicode strings
+        and byte strings.
 
         The default error message can be overridden with the optional
         ``msg`` argument.
-
-        New in Robot Framework 2.7.7.
         """
         if not is_bytes(item):
             self._fail(msg, "'%s' is not a byte string.", item)
