@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -23,12 +24,12 @@ except ImportError:
 
 from robot.api import logger
 from robot.libraries.BuiltIn import BuiltIn
-from robot.utils import (asserts, ET, ETSource, is_string, is_truthy,
+from robot.utils import (asserts, ET, ETSource, is_falsy, is_string, is_truthy,
                          plural_or_not as s)
 from robot.version import get_version
 
 
-should_be_equal = asserts.assert_equals
+should_be_equal = asserts.assert_equal
 should_match = BuiltIn().should_match
 
 
@@ -312,8 +313,9 @@ class XML(object):
     inconvenient especially with xpaths, and by default this library strips
     those namespaces away and moves them to ``xmlns`` attribute instead. That
     can be avoided by passing ``keep_clark_notation`` argument to `Parse XML`
-    keyword. The pros and cons of both approaches are discussed in more detail
-    below.
+    keyword. Alternatively `Parse XML` supports stripping namespace information
+    altogether by using ``strip_namespaces`` argument. The pros and cons of
+    different approaches are discussed in more detail below.
 
     == How ElementTree handles namespaces ==
 
@@ -384,9 +386,20 @@ class XML(object):
 
     == Namespaces when using lxml ==
 
-    Namespaces are handled the same way also when `using lxml`. The only
-    difference is that lxml stores information about namespace prefixes and
-    thus they are preserved if XML is saved.
+    This library handles namespaces same way both when `using lxml` and when
+    not using it. There are, however, differences how lxml internally handles
+    namespaces compared to the standard ElementTree. The main difference is
+    that lxml stores information about namespace prefixes and they are thus
+    preserved if XML is saved. Another visible difference is that lxml includes
+    namespace information in child elements got with `Get Element` if the
+    parent element has namespaces.
+
+    == Stripping namespaces altogether ==
+
+    Because namespaces often add unnecessary complexity, `Parse XML` supports
+    stripping them altogether by using ``strip_namespaces=True``. When this
+    option is enabled, namespaces are not shown anywhere nor are they included
+    if XML is saved.
 
     == Attribute namespaces ==
 
@@ -432,7 +445,6 @@ class XML(object):
 
     ROBOT_LIBRARY_SCOPE = 'GLOBAL'
     ROBOT_LIBRARY_VERSION = get_version()
-    _whitespace = re.compile('\s+')
     _xml_declaration = re.compile('^<\?xml .*\?>')
 
     def __init__(self, use_lxml=False):
@@ -462,8 +474,9 @@ class XML(object):
         if use_lxml and not lxml_etree:
             logger.warn('XML library reverted to use standard ElementTree '
                         'because lxml module is not installed.')
+        self._ns_stripper = NameSpaceStripper(self.etree, self.lxml_etree)
 
-    def parse_xml(self, source, keep_clark_notation=False):
+    def parse_xml(self, source, keep_clark_notation=False, strip_namespaces=False):
         """Parses the given XML file or string into an element structure.
 
         The ``source`` can either be a path to an XML file or a string
@@ -473,22 +486,27 @@ class XML(object):
         instructions in the source XML are removed.
 
         As discussed in `Handling XML namespaces` section, this keyword, by
-        default, strips possible namespaces added by ElementTree into tag names.
-        This typically eases handling XML documents with namespaces
-        considerably. If you do not want that to happen, or want to avoid
-        the small overhead of going through the element structure when your
-        XML does not have namespaces, you can disable this feature by giving
-        ``keep_clark_notation`` argument a true value (see `Boolean arguments`).
+        default, removes namespace information ElementTree has added to tag
+        names and moves it into ``xmlns`` attributes. This typically eases
+        handling XML documents with namespaces considerably. If you do not
+        want that to happen, or want to avoid the small overhead of going
+        through the element structure when your XML does not have namespaces,
+        you can disable this feature by giving ``keep_clark_notation`` argument
+        a true value (see `Boolean arguments`).
+
+        If you want to strip namespace information altogether so that it is
+        not included even if XML is saved, you can give a true value to
+        ``strip_namespaces`` argument. This functionality is new in Robot
+        Framework 3.0.2.
 
         Examples:
         | ${root} = | Parse XML | <root><child/></root> |
-        | ${xml} =  | Parse XML | ${CURDIR}/test.xml    | no namespace cleanup |
+        | ${xml} = | Parse XML | ${CURDIR}/test.xml | keep_clark_notation=True |
+        | ${xml} = | Parse XML | ${CURDIR}/test.xml | strip_namespaces=True |
 
         Use `Get Element` keyword if you want to get a certain element and not
         the whole structure. See `Parsing XML` section for more details and
         examples.
-
-        Stripping namespaces is a new feature in Robot Framework 2.7.5.
         """
         with ETSource(source) as source:
             tree = self.etree.parse(source)
@@ -497,7 +515,7 @@ class XML(object):
             lxml_etree.strip_elements(tree, *strip, **dict(with_tail=False))
         root = tree.getroot()
         if not is_truthy(keep_clark_notation):
-            NameSpaceStripper().strip(root)
+            self._ns_stripper.strip(root, preserve=is_falsy(strip_namespaces))
         return root
 
     def get_element(self, source, xpath='.'):
@@ -590,8 +608,6 @@ class XML(object):
         with `Get Elements` keyword that this keyword uses internally.
 
         See also `Element Should Exist` and `Element Should Not Exist`.
-
-        New in Robot Framework 2.7.5.
         """
         count = len(self.get_elements(source, xpath))
         logger.info("%d element%s matched '%s'." % (count, s(count), xpath))
@@ -607,8 +623,6 @@ class XML(object):
 
         See also `Element Should Not Exist` as well as `Get Element Count`
         that this keyword uses internally.
-
-        New in Robot Framework 2.7.5.
         """
         count = self.get_element_count(source, xpath)
         if not count:
@@ -624,8 +638,6 @@ class XML(object):
 
         See also `Element Should Exist` as well as `Get Element Count`
         that this keyword uses internally.
-
-        New in Robot Framework 2.7.5.
         """
         count = self.get_element_count(source, xpath)
         if count:
@@ -639,11 +651,11 @@ class XML(object):
         keyword.
 
         This keyword returns all the text of the specified element, including
-        all the text its children and grandchildren contains. If the element
+        all the text its children and grandchildren contain. If the element
         has no text, an empty string is returned. The returned text is thus not
         always the same as the `text` attribute of the element.
 
-        Be default all whitespace, including newlines and indentation, inside
+        By default all whitespace, including newlines and indentation, inside
         the element is returned as-is. If ``normalize_whitespace`` is given
         a true value (see `Boolean arguments`), then leading and trailing
         whitespace is stripped, newlines and tabs converted to spaces, and
@@ -678,7 +690,7 @@ class XML(object):
             yield element.tail
 
     def _normalize_whitespace(self, text):
-        return self._whitespace.sub(' ', text.strip())
+        return ' '.join(text.split())
 
     def get_elements_texts(self, source, xpath, normalize_whitespace=False):
         """Returns text of all elements matching ``xpath`` as a list.
@@ -851,8 +863,6 @@ class XML(object):
 
         See also `Get Element Attribute`, `Get Element Attributes`,
         `Element Text Should Be` and `Element Text Should Match`.
-
-        New in Robot Framework 2.7.5.
         """
         attr = self.get_element_attribute(source, name, xpath)
         if attr is not None:
@@ -943,8 +953,6 @@ class XML(object):
 
         Can only set the tag of a single element. Use `Set Elements Tag` to set
         the tag of multiple elements in one call.
-
-        New in Robot Framework 2.7.5.
         """
         source = self.get_element(source)
         self.get_element(source, xpath).tag = tag
@@ -983,8 +991,6 @@ class XML(object):
 
         Can only set the text/tail of a single element. Use `Set Elements Text`
         to set the text/tail of multiple elements in one call.
-
-        New in Robot Framework 2.7.5.
         """
         source = self.get_element(source)
         element = self.get_element(source, xpath)
@@ -1025,8 +1031,6 @@ class XML(object):
 
         Can only set an attribute of a single element. Use `Set Elements
         Attribute` to set an attribute of multiple elements in one call.
-
-        New in Robot Framework 2.7.5.
         """
         if not name:
             raise RuntimeError('Attribute name can not be empty.')
@@ -1063,8 +1067,6 @@ class XML(object):
 
         Can only remove an attribute from a single element. Use `Remove Elements
         Attribute` to remove an attribute of multiple elements in one call.
-
-        New in Robot Framework 2.7.5.
         """
         source = self.get_element(source)
         attrib = self.get_element(source, xpath).attrib
@@ -1100,8 +1102,6 @@ class XML(object):
 
         Can only remove attributes from a single element. Use `Remove Elements
         Attributes` to remove all attributes of multiple elements in one call.
-
-        New in Robot Framework 2.7.5.
         """
         source = self.get_element(source)
         self.get_element(source, xpath).attrib.clear()
@@ -1144,8 +1144,6 @@ class XML(object):
         | Elements Should Be Equal | ${new} | <new id="x"><c1/><c3/><c2/></new> |
 
         Use `Remove Element` or `Remove Elements` to remove elements.
-
-        New in Robot Framework 2.7.5.
         """
         source = self.get_element(source)
         parent = self.get_element(source, xpath)
@@ -1177,8 +1175,6 @@ class XML(object):
         | Element Should Not Exist | ${XML} | xpath=second |
         | Remove Element           | ${XML} | xpath=html/p/b | remove_tail=yes |
         | Element Text Should Be   | ${XML} | Text with italics. | xpath=html/p | normalize_whitespace=yes |
-
-        New in Robot Framework 2.7.5.
         """
         source = self.get_element(source)
         self._remove_element(source, self.get_element(source, xpath), remove_tail)
@@ -1202,8 +1198,6 @@ class XML(object):
         | Remove Elements          | ${XML} | xpath=*/child      |
         | Element Should Not Exist | ${XML} | xpath=second/child |
         | Element Should Not Exist | ${XML} | xpath=third/child  |
-
-        New in Robot Framework 2.7.5.
         """
         source = self.get_element(source)
         for element in self.get_elements(source, xpath):
@@ -1257,8 +1251,6 @@ class XML(object):
         | Elements Should Be Equal | ${XML}   | <example/> |
 
         Use `Remove Element` to remove the whole element.
-
-        New in Robot Framework 2.7.5.
         """
         source = self.get_element(source)
         element = self.get_element(source, xpath)
@@ -1286,8 +1278,6 @@ class XML(object):
         | Elements Should Be Equal | ${elem}  | <first id="1">new text</first> |
         | Elements Should Be Equal | ${copy1} | <first id="new">text</first>   |
         | Elements Should Be Equal | ${copy2} | <first id="1">text</first>     |
-
-        New in Robot Framework 2.7.5.
         """
         return copy.deepcopy(self.get_element(source, xpath))
 
@@ -1343,19 +1333,24 @@ class XML(object):
 
         Use `Element To String` if you just need a string representation of
         the element.
-
-        New in Robot Framework 2.7.5.
         """
         path = os.path.abspath(path.replace('/', os.sep))
         elem = self.get_element(source)
-        if self.lxml_etree:
-            NameSpaceStripper().unstrip(elem)
         tree = self.etree.ElementTree(elem)
-        xml_declaration = {'xml_declaration': True} if self.modern_etree else {}
-        # Need to explicitly open/close files because older ET versions don't
-        # close files they open and Jython/IPY don't close them implicitly.
-        with open(path, 'w') as output:
-            tree.write(output, encoding=encoding, **xml_declaration)
+        config = {'encoding': encoding}
+        if self.modern_etree:
+            config['xml_declaration'] = True
+        if self.lxml_etree:
+            elem = self._ns_stripper.unstrip(elem)
+            # https://bugs.launchpad.net/lxml/+bug/1660433
+            if tree.docinfo.doctype:
+                config['doctype'] = tree.docinfo.doctype
+            tree = self.etree.ElementTree(elem)
+        with open(path, 'wb') as output:
+            if 'doctype' in config:
+                output.write(self.etree.tostring(tree, **config))
+            else:
+                tree.write(output, **config)
         logger.info('XML saved to <a href="file://%s">%s</a>.' % (path, path),
                     html=True)
 
@@ -1388,24 +1383,33 @@ class XML(object):
 
 class NameSpaceStripper(object):
 
-    def strip(self, elem, current_ns=None):
+    def __init__(self, etree, lxml_etree=False):
+        self.etree = etree
+        self.lxml_tree = lxml_etree
+
+    def strip(self, elem, preserve=True, current_ns=None, top=True):
         if elem.tag.startswith('{') and '}' in elem.tag:
             ns, elem.tag = elem.tag[1:].split('}', 1)
-            if ns != current_ns:
+            if preserve and ns != current_ns:
                 elem.attrib['xmlns'] = ns
                 current_ns = ns
         elif current_ns:
             elem.attrib['xmlns'] = ''
             current_ns = None
         for child in elem:
-            self.strip(child, current_ns)
+            self.strip(child, preserve, current_ns, top=False)
+        if top and not preserve and self.lxml_tree:
+            self.etree.cleanup_namespaces(elem)
 
-    def unstrip(self, elem, current_ns=None):
+    def unstrip(self, elem, current_ns=None, copied=False):
+        if not copied:
+            elem = copy.deepcopy(elem)
         ns = elem.attrib.pop('xmlns', current_ns)
         if ns:
             elem.tag = '{%s}%s' % (ns, elem.tag)
         for child in elem:
-            self.unstrip(child, ns)
+            self.unstrip(child, ns, copied=True)
+        return elem
 
 
 class ElementFinder(object):
@@ -1435,8 +1439,8 @@ class ElementFinder(object):
             if not xpath.replace('/', '').isalnum():
                 logger.warn('XPATHs containing non-ASCII characters and '
                             'other than tag names do not always work with '
-                            'Python/Jython versions prior to 2.7. Verify '
-                            'results manually and consider upgrading to 2.7.')
+                            'Python versions prior to 2.7. Verify results '
+                            'manually and consider upgrading to 2.7.')
             return xpath
 
 
