@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -19,7 +20,7 @@ from robot.errors import DataError
 from robot.variables import is_var
 from robot.output import LOGGER
 from robot.writer import DataFileWriter
-from robot.utils import abspath, is_string, normalize, NormalizedDict
+from robot.utils import abspath, is_string, normalize, py2to3, NormalizedDict
 
 from .comments import Comment
 from .populators import FromFilePopulator, FromDirectoryPopulator
@@ -29,17 +30,21 @@ from .settings import (Documentation, Fixture, Timeout, Tags, Metadata,
 
 
 def TestData(parent=None, source=None, include_suites=None,
-             warn_on_skipped=False):
+             warn_on_skipped=False, extensions=None):
     """Parses a file or directory to a corresponding model object.
 
-    :param parent: (optional) parent to be used in creation of the model object.
-    :param source: path where test data is read from.
+    :param parent: Optional parent to be used in creation of the model object.
+    :param source: Path where test data is read from.
+    :param warn_on_skipped: Boolean to control warning about skipped files.
+    :param extensions: List/set of extensions to parse. If None, all files
+        supported by Robot Framework are parsed when searching test cases.
     :returns: :class:`~.model.TestDataDirectory`  if `source` is a directory,
         :class:`~.model.TestCaseFile` otherwise.
     """
     if os.path.isdir(source):
         return TestDataDirectory(parent, source).populate(include_suites,
-                                                          warn_on_skipped)
+                                                          warn_on_skipped,
+                                                          extensions)
     return TestCaseFile(parent, source).populate()
 
 
@@ -48,6 +53,9 @@ class _TestData(object):
     _variable_table_names = 'Variable', 'Variables'
     _testcase_table_names = 'Test Case', 'Test Cases'
     _keyword_table_names = 'Keyword', 'Keywords', 'User Keyword', 'User Keywords'
+    _deprecated = NormalizedDict({'Metadata': 'Settings',
+                                  'User Keyword': 'Keywords',
+                                  'User Keywords': 'Keywords'})
 
     def __init__(self, parent=None, source=None):
         self.parent = parent
@@ -65,7 +73,10 @@ class _TestData(object):
 
     def start_table(self, header_row):
         try:
-            table = self._tables[header_row[0]]
+            name = header_row[0]
+            table = self._tables[name]
+            if name in self._deprecated:
+                self._report_deprecated(name)
         except (KeyError, IndexError):
             return None
         if not self._table_is_allowed(table):
@@ -73,6 +84,12 @@ class _TestData(object):
         table.set_header(header_row)
         return table
 
+    # TODO: Remove support for deprecated tables altogether in RF 3.1.
+    def _report_deprecated(self, name):
+        self.report_invalid_syntax(
+            "Table name '%s' is deprecated. Please use '%s' instead."
+            % (name, self._deprecated[name]), level='WARN')
+
     @property
     def name(self):
         return self._format_name(self._get_basename()) if self.source else None
@@ -204,9 +221,10 @@ class TestDataDirectory(_TestData):
         self.keyword_table = KeywordTable(self)
         _TestData.__init__(self, parent, source)
 
-    def populate(self, include_suites=None, warn_on_skipped=False, recurse=True):
+    def populate(self, include_suites=None, warn_on_skipped=False,
+                 extensions=None, recurse=True):
         FromDirectoryPopulator().populate(self.source, self, include_suites,
-                                          warn_on_skipped, recurse)
+                                          warn_on_skipped, extensions, recurse)
         self.children = [ch for ch in self.children if ch.has_tests()]
         return self
 
@@ -220,9 +238,13 @@ class TestDataDirectory(_TestData):
             return False
         return True
 
-    def add_child(self, path, include_suites):
-        self.children.append(TestData(parent=self,source=path,
-                                      include_suites=include_suites))
+    def add_child(self, path, include_suites, extensions=None,
+                  warn_on_skipped=False):
+        self.children.append(TestData(parent=self,
+                                      source=path,
+                                      include_suites=include_suites,
+                                      extensions=extensions,
+                                      warn_on_skipped=warn_on_skipped))
 
     def has_tests(self):
         return any(ch.has_tests() for ch in self.children)
@@ -232,6 +254,7 @@ class TestDataDirectory(_TestData):
             yield table
 
 
+@py2to3
 class _Table(object):
 
     def __init__(self, parent):
@@ -275,19 +298,34 @@ class _Table(object):
 
 
 class _WithSettings(object):
+    _deprecated = {'document': 'Documentation',
+                   'suiteprecondition': 'Suite Setup',
+                   'suitepostcondition': 'Suite Teardown',
+                   'testprecondition': 'Test Setup',
+                   'testpostcondition': 'Test Teardown',
+                   'precondition': 'Setup',
+                   'postcondition': 'Teardown'}
 
     def get_setter(self, setting_name):
         normalized = self.normalize(setting_name)
+        if normalized in self._deprecated:
+            self._report_deprecated(setting_name, self._deprecated[normalized])
+            normalized = self.normalize(self._deprecated[normalized])
         if normalized in self._setters:
             return self._setters[normalized](self)
         self.report_invalid_syntax("Non-existing setting '%s'." % setting_name)
 
+    def _report_deprecated(self, deprecated, use_instead):
+         self.report_invalid_syntax(
+             "Setting '%s' is deprecated. Use '%s' instead."
+             % (deprecated.rstrip(':'), use_instead), level='WARN')
+
     def is_setting(self, setting_name):
         return self.normalize(setting_name) in self._setters
 
     def normalize(self, setting):
         result = normalize(setting)
-        return result[0:-1] if result and result[-1]==':' else result
+        return result[:-1] if result[-1:] == ':' else result
 
 
 class _SettingTable(_Table, _WithSettings):
@@ -334,15 +372,10 @@ class _SettingTable(_Table, _WithSettings):
 class TestCaseFileSettingTable(_SettingTable):
 
     _setters = {'documentation': lambda s: s.doc.populate,
-                'document': lambda s: s.doc.populate,
                 'suitesetup': lambda s: s.suite_setup.populate,
-                'suiteprecondition': lambda s: s.suite_setup.populate,
                 'suiteteardown': lambda s: s.suite_teardown.populate,
-                'suitepostcondition': lambda s: s.suite_teardown.populate,
                 'testsetup': lambda s: s.test_setup.populate,
-                'testprecondition': lambda s: s.test_setup.populate,
                 'testteardown': lambda s: s.test_teardown.populate,
-                'testpostcondition': lambda s: s.test_teardown.populate,
                 'forcetags': lambda s: s.force_tags.populate,
                 'defaulttags': lambda s: s.default_tags.populate,
                 'testtemplate': lambda s: s.test_template.populate,
@@ -363,7 +396,6 @@ class TestCaseFileSettingTable(_SettingTable):
 class ResourceFileSettingTable(_SettingTable):
 
     _setters = {'documentation': lambda s: s.doc.populate,
-                'document': lambda s: s.doc.populate,
                 'library': lambda s: s.imports.populate_library,
                 'resource': lambda s: s.imports.populate_resource,
                 'variables': lambda s: s.imports.populate_variables}
@@ -376,15 +408,10 @@ class ResourceFileSettingTable(_SettingTable):
 class InitFileSettingTable(_SettingTable):
 
     _setters = {'documentation': lambda s: s.doc.populate,
-                'document': lambda s: s.doc.populate,
                 'suitesetup': lambda s: s.suite_setup.populate,
-                'suiteprecondition': lambda s: s.suite_setup.populate,
                 'suiteteardown': lambda s: s.suite_teardown.populate,
-                'suitepostcondition': lambda s: s.suite_teardown.populate,
                 'testsetup': lambda s: s.test_setup.populate,
-                'testprecondition': lambda s: s.test_setup.populate,
                 'testteardown': lambda s: s.test_teardown.populate,
-                'testpostcondition': lambda s: s.test_teardown.populate,
                 'testtimeout': lambda s: s.test_timeout.populate,
                 'forcetags': lambda s: s.force_tags.populate,
                 'library': lambda s: s.imports.populate_library,
@@ -417,6 +444,7 @@ class VariableTable(_Table):
         return iter(self.variables)
 
 
+@py2to3
 class TestCaseTable(_Table):
     type = 'test case'
 
@@ -461,6 +489,7 @@ class KeywordTable(_Table):
         return iter(self.keywords)
 
 
+@py2to3
 class Variable(object):
 
     def __init__(self, parent, name, value, comment=None):
@@ -522,12 +551,9 @@ class TestCase(_WithSteps, _WithSettings):
         self.steps = []
 
     _setters = {'documentation': lambda s: s.doc.populate,
-                'document': lambda s: s.doc.populate,
                 'template': lambda s: s.template.populate,
                 'setup': lambda s: s.setup.populate,
-                'precondition': lambda s: s.setup.populate,
                 'teardown': lambda s: s.teardown.populate,
-                'postcondition': lambda s: s.teardown.populate,
                 'tags': lambda s: s.tags.populate,
                 'timeout': lambda s: s.timeout.populate}
 
@@ -577,7 +603,6 @@ class UserKeyword(TestCase):
         self.steps = []
 
     _setters = {'documentation': lambda s: s.doc.populate,
-                'document': lambda s: s.doc.populate,
                 'arguments': lambda s: s.args.populate,
                 'return': lambda s: s.return_.populate,
                 'timeout': lambda s: s.timeout.populate,
@@ -645,19 +670,16 @@ class ForLoop(_WithSteps):
 class Step(object):
 
     def __init__(self, content, comment=None):
-        self.assign = list(self._get_assigned_vars(content))
-        try:
-            self.name = content[len(self.assign)]
-        except IndexError:
-            self.name = None
-        self.args = content[len(self.assign)+1:]
+        self.assign = self._get_assign(content)
+        self.name = content.pop(0) if content else None
+        self.args = content
         self.comment = Comment(comment)
 
-    def _get_assigned_vars(self, content):
-        for item in content:
-            if not is_var(item.rstrip('= ')):
-                return
-            yield item
+    def _get_assign(self, content):
+        assign = []
+        while content and is_var(content[0].rstrip('= ')):
+            assign.append(content.pop(0))
+        return assign
 
     def is_comment(self):
         return not (self.assign or self.name or self.args)
