@@ -1,4 +1,5 @@
-#  Copyright 2008-2014 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -23,6 +24,12 @@ class OutputWriter(XmlLogger):
     def start_message(self, msg):
         self._write_message(msg)
 
+    def visit_keyword(self, kw):
+        self.start_keyword(kw)
+        for child in kw.children:
+            child.visit(self)
+        self.end_keyword(kw)
+
     def close(self):
         self._writer.end('robot')
         self._writer.close()
