@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -14,7 +15,7 @@
 
 from robot.errors import DataError
 from robot.model import SuiteVisitor
-from robot.utils import ET, ETSource, get_error_message
+from robot.utils import ET, ETSource, get_error_message, unic
 
 from .executionresult import Result, CombinedResult
 from .flattenkeywordmatcher import (FlattenByNameMatcher, FlattenByTypeMatcher,
@@ -26,14 +27,16 @@ from .xmlelementhandlers import XmlElementHandler
 def ExecutionResult(*sources, **options):
     """Factory method to constructs :class:`~.executionresult.Result` objects.
 
-    :param sources: Path(s) to output XML file(s).
-    :param options: Configuration options. ``merge`` with True value causes
-                    multiple results to be combined so that tests in the latter
-                    results replace the ones in the original. Other options
-                    are passed further to :py:class:`~ExecutionResultBuilder`.
+    :param sources: Path(s) to the XML output file(s).
+    :param options: Configuration options.
+        Using ``merge=True`` causes multiple results to be combined so that
+        tests in the latter results replace the ones in the original. Other
+        options are passed directly to the :class:`ExecutionResultBuilder`
+        object used internally.
     :returns: :class:`~.executionresult.Result` instance.
 
-    See :mod:`~robot.result` package for a usage example.
+    Should be imported by external code via the :mod:`robot.api` package.
+    See the :mod:`robot.result` package for a usage example.
     """
     if not sources:
         raise DataError('One or more data source needed.')
@@ -65,18 +68,26 @@ def _single_result(source, options):
         error = err.strerror
     except:
         error = get_error_message()
-    raise DataError("Reading XML source '%s' failed: %s" % (unicode(ets), error))
+    raise DataError("Reading XML source '%s' failed: %s" % (unic(ets), error))
 
 
 class ExecutionResultBuilder(object):
+    """Builds :class:`~.executionresult.Result` objects based on output files.
+
+    Instead of using this builder directly, it is recommended to use the
+    :func:`ExecutionResult` factory method.
+    """
 
     def __init__(self, source, include_keywords=True, flattened_keywords=None):
-        """Builds :class:`~.executionresult.Result` objects from existing
-        output XML files on the file system.
-
-        :param source: Path to output XML file.
-        :param include_keywords: Include keyword information to the
-            :class:`~.executionresult.Result` objects
+        """
+        :param source: Path to the XML output file to build
+            :class:`~.executionresult.Result` objects from.
+        :param include_keywords: Boolean controlling whether to include
+            keyword information in the result or not. Keywords are
+            not needed when generating only report.
+        :param flatten_keywords: List of patterns controlling what keywords to
+            flatten. See the documentation of ``--flattenkeywords`` option for
+            more details.
         """
         self._source = source \
             if isinstance(source, ETSource) else ETSource(source)
