@@ -1,4 +1,5 @@
-#  Copyright 2008-2013 Nokia Siemens Networks Oyj
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -14,17 +15,21 @@
 
 import sys
 
+from .argumentmapper import ArgumentMapper
+from .argumentresolver import ArgumentResolver
+
 
 class ArgumentSpec(object):
 
-    def __init__(self, name, type='Keyword', positional=None, defaults=None,
-                 varargs=None, kwargs=None):
+    def __init__(self, name=None, type='Keyword', positional=None,
+                 defaults=None, varargs=None, kwargs=None, supports_named=True):
         self.name = name
         self.type = type
         self.positional = positional or []
         self.defaults = defaults or []
         self.varargs = varargs
         self.kwargs = kwargs
+        self.supports_named = supports_named
 
     @property
     def minargs(self):
@@ -32,4 +37,14 @@ class ArgumentSpec(object):
 
     @property
     def maxargs(self):
-        return len(self.positional) if not self.varargs else sys.maxint
+        return len(self.positional) if not self.varargs else sys.maxsize
+
+    def resolve(self, arguments, variables=None, resolve_named=True,
+                resolve_variables_until=None, dict_to_kwargs=False):
+        resolver = ArgumentResolver(self, resolve_named,
+                                    resolve_variables_until, dict_to_kwargs)
+        return resolver.resolve(arguments, variables)
+
+    def map(self, positional, named, replace_defaults=True):
+        mapper = ArgumentMapper(self)
+        return mapper.map(positional, named, replace_defaults)
