@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -14,13 +15,14 @@
 
 import os
 import copy
+from itertools import chain
 
 from robot.errors import DataError
-from robot.libraries import STDLIBS, STDLIB_TO_DEPRECATED_MAP
+from robot.libraries import STDLIBS
 from robot.output import LOGGER, Message
 from robot.parsing.settings import Library, Variables, Resource
-from robot.utils import (eq, find_file, is_string, printable_name, seq2str2,
-                         RecommendationFinder)
+from robot.utils import (eq, find_file, is_string, OrderedDict, printable_name,
+                         seq2str2, RecommendationFinder)
 
 from .usererrorhandler import UserErrorHandler
 from .userkeyword import UserLibrary
@@ -35,15 +37,14 @@ class Namespace(object):
     _default_libraries = ('BuiltIn', 'Reserved', 'Easter')
     _library_import_by_path_endings = ('.py', '.java', '.class', '/', os.sep)
 
-    def __init__(self, variables, suite, user_keywords, imports):
+    def __init__(self, variables, suite, resource):
         LOGGER.info("Initializing namespace for test suite '%s'" % suite.longname)
-        self.suite = suite
-        self.test = None
-        self.uk_handlers = []
         self.variables = variables
-        self._imports = imports
-        self._kw_store = KeywordStore(user_keywords, suite.source)
+        self._imports = resource.imports
+        self._kw_store = KeywordStore(resource)
         self._imported_variable_files = ImportCache()
+        self._suite_name = suite.longname
+        self._running_test = False
 
     @property
     def libraries(self):
@@ -64,7 +65,7 @@ class Namespace(object):
                     raise DataError('%s setting requires a name' % item.type)
                 self._import(item)
             except DataError as err:
-                item.report_invalid_syntax(unicode(err))
+                item.report_invalid_syntax(err.message)
 
     def _import(self, import_setting):
         action = {'Library': self._import_library,
@@ -81,7 +82,7 @@ class Namespace(object):
         if overwrite or path not in self._kw_store.resources:
             resource = IMPORTER.import_resource(path)
             self.variables.set_from_variable_table(resource.variables, overwrite)
-            user_library = UserLibrary(resource.keywords, resource.source)
+            user_library = UserLibrary(resource)
             self._kw_store.resources[path] = user_library
             self._handle_imports(resource.imports)
             LOGGER.imported("Resource", user_library.name,
@@ -89,7 +90,7 @@ class Namespace(object):
                             source=path)
         else:
             LOGGER.info("Resource file '%s' already imported by suite '%s'"
-                        % (path, self.suite.longname))
+                        % (path, self._suite_name))
 
     def _validate_not_importing_init_file(self, path):
         name = os.path.splitext(os.path.basename(path))[0]
@@ -115,7 +116,7 @@ class Namespace(object):
             if args:
                 msg += " with arguments %s" % seq2str2(args)
             LOGGER.info("%s already imported by suite '%s'"
-                        % (msg, self.suite.longname))
+                        % (msg, self._suite_name))
 
     def import_library(self, name, args=None, alias=None, notify=True):
         self._import_library(Library(None, name, args=args, alias=alias),
@@ -127,7 +128,7 @@ class Namespace(object):
                                       import_setting.alias, self.variables)
         if lib.name in self._kw_store.libraries:
             LOGGER.info("Test library '%s' already imported by suite '%s'"
-                        % (lib.name, self.suite.longname))
+                        % (lib.name, self._suite_name))
             return
         if notify:
             LOGGER.imported("Library", lib.name,
@@ -137,10 +138,8 @@ class Namespace(object):
                             source=lib.source)
         self._kw_store.libraries[lib.name] = lib
         lib.start_suite()
-        if self.test:
+        if self._running_test:
             lib.start_test()
-        if name in STDLIB_TO_DEPRECATED_MAP:
-            self.import_library(STDLIB_TO_DEPRECATED_MAP[name], notify=False)
 
     def _resolve_name(self, import_setting):
         name = import_setting.name
@@ -148,16 +147,25 @@ class Namespace(object):
             name = self.variables.replace_string(name)
         except DataError as err:
             self._raise_replacing_vars_failed(import_setting, err)
-        return self._get_name(name, import_setting.directory, import_setting.type)
+        return self._get_name(name, import_setting)
 
     def _raise_replacing_vars_failed(self, import_setting, err):
         raise DataError("Replacing variables from setting '%s' failed: %s"
-                        % (import_setting.type, unicode(err)))
+                        % (import_setting.type, err.message))
 
-    def _get_name(self, name, basedir, import_type):
-        if import_type == 'Library' and not self._is_library_by_path(name):
-            return name.replace(' ', '')
-        return find_file(name, basedir, file_type=import_type)
+    def _get_name(self, name, import_setting):
+        if import_setting.type == 'Library' and not self._is_library_by_path(name):
+            if ' ' in name:
+                # TODO: Remove support for extra spaces in name in RF 3.1.
+                # https://github.com/robotframework/robotframework/issues/2264
+                warning = ("Importing library with extra spaces in name like "
+                           "'%s' is deprecated. Remove spaces and use '%s' "
+                           "instead." % (name, name.replace(' ', '')))
+                import_setting.report_invalid_syntax(warning, 'WARN')
+                name = name.replace(' ', '')
+            return name
+        return find_file(name, import_setting.directory,
+                         file_type=import_setting.type)
 
     def _is_library_by_path(self, path):
         return path.lower().endswith(self._library_import_by_path_endings)
@@ -173,35 +181,31 @@ class Namespace(object):
         self._kw_store.search_order = new_order
         return old_order
 
-    def start_test(self, test):
-        self.test = test
+    def start_test(self):
+        self._running_test = True
         self.variables.start_test()
         for lib in self.libraries:
             lib.start_test()
 
     def end_test(self):
-        self.test = None
         self.variables.end_test()
-        self.uk_handlers = []
         for lib in self.libraries:
             lib.end_test()
+        self._running_test = True
 
     def start_suite(self):
         self.variables.start_suite()
 
     def end_suite(self):
-        self.suite = None
         self.variables.end_suite()
         for lib in self.libraries:
             lib.end_suite()
 
-    def start_user_keyword(self, handler):
+    def start_user_keyword(self):
         self.variables.start_keyword()
-        self.uk_handlers.append(handler)
 
     def end_user_keyword(self):
         self.variables.end_keyword()
-        self.uk_handlers.pop()
 
     def get_library_instance(self, libname):
         return self._kw_store.get_library(libname).get_instance()
@@ -215,25 +219,19 @@ class Namespace(object):
         library.reload()
         return library
 
-    def get_handler(self, name):
+    def get_runner(self, name):
         try:
-            handler = self._kw_store.get_handler(name)
+            return self._kw_store.get_runner(name)
         except DataError as err:
-            handler = UserErrorHandler(name, unicode(err))
-        self._replace_variables_from_user_handlers(handler)
-        return handler
-
-    def _replace_variables_from_user_handlers(self, handler):
-        if hasattr(handler, 'replace_variables'):
-            handler.replace_variables(self.variables)
+            return UserErrorHandler(name, err.message)
 
 
 class KeywordStore(object):
 
-    def __init__(self, suite_keywords, suite_source):
-        self.user_keywords = UserLibrary(suite_keywords, suite_source,
+    def __init__(self, resource):
+        self.user_keywords = UserLibrary(resource,
                                          UserLibrary.TEST_CASE_FILE_TYPE)
-        self.libraries = {}
+        self.libraries = OrderedDict()
         self.resources = ImportCache()
         self.search_order = ()
 
@@ -263,11 +261,11 @@ class KeywordStore(object):
                 return lib
         self._no_library_found(instance)
 
-    def get_handler(self, name):
-        handler = self._get_handler(name)
-        if handler is None:
+    def get_runner(self, name):
+        runner = self._get_runner(name)
+        if runner is None:
             self._raise_no_keyword_found(name)
-        return handler
+        return runner
 
     def _raise_no_keyword_found(self, name):
         msg = "No keyword with name '%s' found." % name
@@ -278,91 +276,80 @@ class KeywordStore(object):
         msg = finder.format_recommendations(msg, recommendations)
         raise DataError(msg)
 
-    def _get_handler(self, name):
-        handler = None
+    def _get_runner(self, name):
         if not name:
             raise DataError('Keyword name cannot be empty.')
         if not is_string(name):
             raise DataError('Keyword name must be a string.')
-        if '.' in name:
-            handler = self._get_explicit_handler(name)
-        if not handler:
-            handler = self._get_implicit_handler(name)
-        if not handler:
-            handler = self._get_bdd_style_handler(name)
-        return handler
+        runner = self._get_runner_from_test_case_file(name)
+        if not runner and '.' in name:
+            runner = self._get_explicit_runner(name)
+        if not runner:
+            runner = self._get_implicit_runner(name)
+        if not runner:
+            runner = self._get_bdd_style_runner(name)
+        return runner
 
-    def _get_bdd_style_handler(self, name):
+    def _get_bdd_style_runner(self, name):
         for prefix in ['given ', 'when ', 'then ', 'and ', 'but ']:
             if name.lower().startswith(prefix):
-                handler = self._get_handler(name[len(prefix):])
-                if handler:
-                    handler = copy.copy(handler)
-                    handler.name = name
-                return handler
+                runner = self._get_runner(name[len(prefix):])
+                if runner:
+                    runner = copy.copy(runner)
+                    runner.name = name
+                return runner
         return None
 
-    def _get_implicit_handler(self, name):
-        for method in [self._get_handler_from_test_case_file_user_keywords,
-                       self._get_handler_from_resource_file_user_keywords,
-                       self._get_handler_from_library_keywords]:
-            handler = method(name)
-            if handler:
-                return handler
-        return None
+    def _get_implicit_runner(self, name):
+        runner = self._get_runner_from_resource_files(name)
+        if not runner:
+            runner = self._get_runner_from_libraries(name)
+        return runner
 
-    def _get_handler_from_test_case_file_user_keywords(self, name):
+    def _get_runner_from_test_case_file(self, name):
         if name in self.user_keywords.handlers:
-            return self.user_keywords.handlers[name]
+            return self.user_keywords.handlers.create_runner(name)
 
-    def _get_handler_from_resource_file_user_keywords(self, name):
-        found = [lib.handlers[name] for lib in self.resources.values()
+    def _get_runner_from_resource_files(self, name):
+        found = [lib.handlers.create_runner(name)
+                 for lib in self.resources.values()
                  if name in lib.handlers]
         if not found:
             return None
         if len(found) > 1:
-            found = self._get_handler_based_on_search_order(found)
+            found = self._get_runner_based_on_search_order(found)
         if len(found) == 1:
             return found[0]
         self._raise_multiple_keywords_found(name, found)
 
-    def _get_handler_from_library_keywords(self, name):
-        found = [lib.handlers[name] for lib in self.libraries.values()
+    def _get_runner_from_libraries(self, name):
+        found = [lib.handlers.create_runner(name) for lib in self.libraries.values()
                  if name in lib.handlers]
         if not found:
             return None
         if len(found) > 1:
-            found = self._get_handler_based_on_search_order(found)
+            found = self._get_runner_based_on_search_order(found)
         if len(found) == 2:
-            found = self._prefer_process_over_operatingsystem(*found)
-        if len(found) == 2:
-            found = self._filter_stdlib_handler(*found)
+            found = self._filter_stdlib_runner(*found)
         if len(found) == 1:
             return found[0]
         self._raise_multiple_keywords_found(name, found)
 
-    def _get_handler_based_on_search_order(self, handlers):
+    def _get_runner_based_on_search_order(self, runners):
         for libname in self.search_order:
-            for handler in handlers:
-                if eq(libname, handler.libname):
-                    return [handler]
-        return handlers
+            for runner in runners:
+                if eq(libname, runner.libname):
+                    return [runner]
+        return runners
 
-    def _prefer_process_over_operatingsystem(self, handler1, handler2):
-        handlers = {handler1.library.orig_name: handler1,
-                    handler2.library.orig_name: handler2}
-        if set(handlers) == set(['Process', 'OperatingSystem']):
-            return [handlers['Process']]
-        return [handler1, handler2]
-
-    def _filter_stdlib_handler(self, handler1, handler2):
+    def _filter_stdlib_runner(self, runner1, runner2):
         stdlibs_without_remote = STDLIBS - set(['Remote'])
-        if handler1.library.orig_name in stdlibs_without_remote:
-            standard, custom = handler1, handler2
-        elif handler2.library.orig_name in stdlibs_without_remote:
-            standard, custom = handler2, handler1
+        if runner1.library.orig_name in stdlibs_without_remote:
+            standard, custom = runner1, runner2
+        elif runner2.library.orig_name in stdlibs_without_remote:
+            standard, custom = runner2, runner1
         else:
-            return [handler1, handler2]
+            return [runner1, runner2]
         if not RUN_KW_REGISTER.is_run_keyword(custom.library.orig_name, custom.name):
             self._custom_and_standard_keyword_conflict_warning(custom, standard)
         return [custom]
@@ -386,7 +373,7 @@ class KeywordStore(object):
         else:
             custom.pre_run_messages = [warning]
 
-    def _get_explicit_handler(self, name):
+    def _get_explicit_runner(self, name):
         found = []
         for owner_name, kw_name in self._yield_owner_and_kw_names(name):
             found.extend(self._find_keywords(owner_name, kw_name))
@@ -400,15 +387,15 @@ class KeywordStore(object):
             yield '.'.join(tokens[:i]), '.'.join(tokens[i:])
 
     def _find_keywords(self, owner_name, name):
-        return [owner.handlers[name]
-                for owner in self.libraries.values() + self.resources.values()
+        return [owner.handlers.create_runner(name)
+                for owner in chain(self.libraries.values(), self.resources.values())
                 if eq(owner.name, owner_name) and name in owner.handlers]
 
     def _raise_multiple_keywords_found(self, name, found, implicit=True):
         error = "Multiple keywords with name '%s' found" % name
         if implicit:
             error += ". Give the full name of the keyword you want to use"
-        names = sorted(handler.longname for handler in found)
+        names = sorted(runner.longname for runner in found)
         raise DataError('\n    '.join([error+':'] + names))
 
 
@@ -440,21 +427,13 @@ class KeywordRecommendationFinder(object):
         return names
 
     def _get_all_handler_names(self):
-        """Return a list of (library name, handler name) tuples.
-
-        For user keywords, library name == None.
-
-        Excludes DeprecatedBuiltIn, DeprecatedOperatingSystem,
-        and Reserved libraries.
-        """
-        excluded = ['DeprecatedBuiltIn', 'DeprecatedOperatingSystem',
-                    'Reserved']
-        handlers = [(None, printable_name(handler.name, True))
+        """Return a list of `(library_name, handler_name)` tuples."""
+        handlers = [('', printable_name(handler.name, True))
                     for handler in self.user_keywords.handlers]
-        for library in (self.libraries.values() + self.resources.values()):
-            if library.name not in excluded:
+        for library in chain(self.libraries.values(), self.resources.values()):
+            if library.name != 'Reserved':
                 handlers.extend(
-                    ((library.name,
+                    ((library.name or '',
                       printable_name(handler.name, code_style=True))
                      for handler in library.handlers))
         # sort handlers to ensure consistent ordering between Jython and Python
