@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -25,7 +26,8 @@ from robot.errors import DataError, Information, FrameworkError
 from robot.version import get_full_version
 
 from .misc import plural_or_not
-from .encoding import decode_output, decode_from_system
+from .encoding import console_decode, system_decode
+from .platform import PY2
 from .utf8reader import Utf8Reader
 from .robottypes import is_falsy, is_integer, is_list_like, is_string, is_unicode
 
@@ -40,7 +42,7 @@ ESCAPES = dict(
 
 
 def cmdline2list(args, escaping=False):
-    if is_unicode(args):
+    if PY2 and is_unicode(args):
         args = args.encode('UTF-8')
         decode = lambda item: item.decode('UTF-8')
     else:
@@ -141,18 +143,27 @@ class ArgumentParser(object):
         amount of horizontal space as <---ESCAPES--->. Both help and version
         are wrapped to Information exception.
         """
-        if self._env_options:
-            args = cmdline2list(os.getenv(self._env_options, '')) + list(args)
-        args = [decode_from_system(a) for a in args]
+        args = self._get_env_options() + list(args)
+        args = [system_decode(a) for a in args]
         if self._auto_argumentfile:
             args = self._process_possible_argfile(args)
         opts, args = self._parse_args(args)
+        if self._auto_argumentfile and opts.get('argumentfile'):
+            raise DataError("Using '--argumentfile' option in shortened format "
+                            "like '--argumentf' is not supported.")
         opts, args = self._handle_special_options(opts, args)
         self._arg_limit_validator(args)
         if self._validator:
             opts, args = self._validator(opts, args)
         return opts, args
 
+    def _get_env_options(self):
+        if self._env_options:
+            options = os.getenv(self._env_options)
+            if options:
+                return cmdline2list(options)
+        return []
+
     def _handle_special_options(self, opts, args):
         if self._auto_escape and opts.get('escape'):
             opts, args = self._unescape_opts_and_args(opts, args)
@@ -332,7 +343,7 @@ class ArgumentParser(object):
             if drive:
                 ret.append(drive)
                 drive = ''
-            if len(item) == 1 and item in string.letters:
+            if len(item) == 1 and item in string.ascii_letters:
                 drive = item
             else:
                 ret.append(item)
@@ -409,11 +420,12 @@ class ArgFileParser(object):
         for opt in self._options:
             start = opt + '=' if opt.startswith('--') else opt
             for index, arg in enumerate(args):
+                normalized_arg = arg.lower() if opt.startswith('--') else arg
                 # Handles `--argumentfile foo` and `-A foo`
-                if arg == opt and index + 1 < len(args):
+                if normalized_arg == opt and index + 1 < len(args):
                     return args[index+1], slice(index, index+2)
                 # Handles `--argumentfile=foo` and `-Afoo`
-                if arg.startswith(start):
+                if normalized_arg.startswith(start):
                     return arg[len(start):], slice(index, index+1)
         return None, -1
 
@@ -433,7 +445,7 @@ class ArgFileParser(object):
                             % (path, err))
 
     def _read_from_stdin(self):
-        return decode_output(sys.__stdin__.read())
+        return console_decode(sys.__stdin__.read())
 
     def _process_file(self, content):
         args = []
