@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -15,15 +16,31 @@
 import os
 import os.path
 import sys
-import urllib
 
 from robot.errors import DataError
 
-from .encoding import decode_from_system
-from .platform import WINDOWS
+from .encoding import system_decode
+from .platform import WINDOWS, PY2
 from .robottypes import is_unicode
+from .unic import unic
 
 
+if sys.version_info < (2,7):
+    def _abspath(path):
+        if WINDOWS and os.path.splitunc(path)[0]:
+            return os.path.abspath(path)
+        return os.path.abspath(os.path.join(os.getcwdu(), path))
+else:
+    _abspath = os.path.abspath
+
+if PY2:
+    from urllib import pathname2url
+
+    def path_to_url(path):
+        return pathname2url(path.encode('UTF-8'))
+else:
+    from urllib.request import pathname2url as path_to_url
+
 if WINDOWS:
     CASE_INSENSITIVE_FILESYSTEM = True
 else:
@@ -36,13 +53,15 @@ else:
 def normpath(path, case_normalize=False):
     """Replacement for os.path.normpath with some enhancements.
 
-    1. Non-Unicode paths are converted to Unicode using file system encoding.
-    2. Optionally lower-case paths on case-insensitive file systems.
+    1. Convert non-Unicode paths to Unicode using the file system encoding.
+    2. NFC normalize Unicode paths (affects mainly OSX).
+    3. Optionally lower-case paths on case-insensitive file systems.
        That includes Windows and also OSX in default configuration.
-    3. Turn ``c:`` into ``c:\\`` on Windows instead of keeping it as ``c:``.
+    4. Turn ``c:`` into ``c:\\`` on Windows instead of keeping it as ``c:``.
     """
     if not is_unicode(path):
-        path = decode_from_system(path)
+        path = system_decode(path)
+    path = unic(path)  # Handles NFC normalization on OSX
     path = os.path.normpath(path)
     if case_normalize and CASE_INSENSITIVE_FILESYSTEM:
         path = path.lower()
@@ -62,38 +81,33 @@ def abspath(path, case_normalize=False):
        http://bugs.python.org/issue3426
     """
     path = normpath(path, case_normalize)
-    if os.path.isabs(path):
-        return path
-    return normpath(os.path.join(os.getcwdu(), path), case_normalize)
+    return normpath(_abspath(path), case_normalize)
 
 
-# TODO: Investigate could this be replaced with os.path.relpath in RF 2.9.
 def get_link_path(target, base):
-    """Returns a relative path to a target from a base.
+    """Returns a relative path to ``target`` from ``base``.
 
-    If base is an existing file, then its parent directory is considered.
-    Otherwise, base is assumed to be a directory.
+    If ``base`` is an existing file, then its parent directory is considered to
+    be the base. Otherwise ``base`` is assumed to be a directory.
 
-    Rationale: os.path.relpath is not available before Python 2.6
+    The returned path is URL encoded. On Windows returns an absolute path with
+    ``file:`` prefix if the target is on a different drive.
     """
-    path =  _get_pathname(target, base)
-    url = urllib.pathname2url(path.encode('UTF-8'))
+    path = _get_link_path(target, base)
+    url = path_to_url(path)
     if os.path.isabs(path):
         url = 'file:' + url
-    # At least Jython seems to use 'C|/Path' and not 'C:/Path'
-    if os.sep == '\\' and '|/' in url:
-        url = url.replace('|/', ':/', 1)
-    return url.replace('%5C', '/').replace('%3A', ':').replace('|', ':')
+    return url
 
-def _get_pathname(target, base):
+def _get_link_path(target, base):
     target = abspath(target)
     base = abspath(base)
     if os.path.isfile(base):
         base = os.path.dirname(base)
     if base == target:
-        return os.path.basename(target)
+        return '.'
     base_drive, base_path = os.path.splitdrive(base)
-    # if in Windows and base and link on different drives
+    # Target and base on different drives
     if os.path.splitdrive(target)[0] != base_drive:
         return target
     common_len = len(_common_path(base, target))
@@ -102,7 +116,8 @@ def _get_pathname(target, base):
     if common_len == len(base_drive) + len(os.sep):
         common_len -= len(os.sep)
     dirs_up = os.sep.join([os.pardir] * base[common_len:].count(os.sep))
-    return os.path.join(dirs_up, target[common_len + len(os.sep):])
+    path = os.path.join(dirs_up, target[common_len + len(os.sep):])
+    return os.path.normpath(path)
 
 def _common_path(p1, p2):
     """Returns the longest path common to p1 and p2.
@@ -123,18 +138,37 @@ def _common_path(p1, p2):
 
 def find_file(path, basedir='.', file_type=None):
     path = os.path.normpath(path.replace('/', os.sep))
-    for base in [basedir] + sys.path:
-        if not (base and os.path.isdir(base)):
-            continue
-        if not is_unicode(base):
-            base = decode_from_system(base)
-        ret = os.path.abspath(os.path.join(base, path))
-        if os.path.isfile(ret):
-            return ret
-        if os.path.isdir(ret) and os.path.isfile(os.path.join(ret, '__init__.py')):
-            return ret
+    if os.path.isabs(path):
+        ret = _find_absolute_path(path)
+    else:
+        ret = _find_relative_path(path, basedir)
+    if ret:
+        return ret
     default = file_type or 'File'
     file_type = {'Library': 'Test library',
                  'Variables': 'Variable file',
                  'Resource': 'Resource file'}.get(file_type, default)
     raise DataError("%s '%s' does not exist." % (file_type, path))
+
+
+def _find_absolute_path(path):
+    if _is_valid_file(path):
+        return path
+    return None
+
+
+def _find_relative_path(path, basedir):
+    for base in [basedir] + sys.path:
+        if not (base and os.path.isdir(base)):
+            continue
+        if not is_unicode(base):
+            base = system_decode(base)
+        ret = os.path.abspath(os.path.join(base, path))
+        if _is_valid_file(ret):
+            return ret
+    return None
+
+
+def _is_valid_file(path):
+    return os.path.isfile(path) or \
+        (os.path.isdir(path) and os.path.isfile(os.path.join(path, '__init__.py')))
