@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -124,7 +125,7 @@ class ExtendedFinder(object):
             variable = self._find_variable('${%s}' % base_name)
         except DataError as err:
             raise VariableError("Resolving variable '%s' failed: %s"
-                                % (name, unicode(err)))
+                                % (name, err.message))
         try:
             return eval('_BASE_VAR_' + extended, {'_BASE_VAR_': variable})
         except:
